/*
 *  Chart widget using OpenLayers that provides compatibility
 *  with the original ECMWF chart widget, for example for sending
 *  and receiving bus messages. Compatibility may be removed
 *  in the future.
 */
import Bus from 'bus'
import Product from 'chart/product'
import mixins from 'mixins'

// http://openlayers.org/en/latest/apidoc/
import Map from 'ol/map'
import View from 'ol/view'
import Tile from 'ol/layer/tile'
import TileWMS from 'ol/source/tilewms'

var Chart = function(args) {
	var self = this;

	var o = this.options = extend({
		product: null,
		bus: Bus()
	}, args);

	this.product = o.product

	o.bus
		.listen('layer-delete', function(_, msg) {
			self.remove_layer(msg.layer_id)
		})
		.listen('layer-options-change', function(_, msg) {
		})

	this.map = new Map({
		target: 'map',
		layers: this.product.state.layers.map(function(v) {
			return new Tile({
				source: new TileWMS({
					url: 'https://apps.ecmwf.int/wms',
					params: {LAYERS: v.name, token: 'metview'}
				})
			})
		}),
		view: new View({
			projection: 'EPSG:4326',
			center: [0, 0],
			zoom: 3
		})
	})
}

Chart.prototype = extend({
	add_layer: function(layer) {
	},

	remove_layer: function(layer_id) {
		var layer = this.product.layers_by_id[layer_id];
		var index = this.product.layerList().indexOf(layer)

		if(index > -1){
			this._push_undo();
			this.product.layerDelete(layer_id);
			this._clear()._redraw(function() {
				this._notifyProductChange();
			});

			var l = this.map.getLayers().item(index)
			if(l) this.map.removeLayer(l)
		}
	},

	/*
	 *  Change to named product. Is very slow because an extra plot request is made to get the layer list.
	 */
	set_product: function(p) {
		var self = this;
		var old_layers = this.product.layerList()

		(this.product = new Product())
			.init({product: p.name()})
			.get_chart({
				json: {
					product: this.product.as_json({width: 128, height: 64}, this.time, this.mail, 'png')
				},
				success: function(product, time) {
					product = product.product
					var map_layers = self.map.getLayers()
					var new_layers = self.product.layerList()

					// Truncate the layerlist if it is too long
					if(map_layers.getLength() > new_layers.length){
						for(var i = map_layers.getLength() - 1; i >= new_layers.length; i--)
							map_layers.removeAt(i)
					}

					product.layers.forEach(function(v, i) {
						if((!old_layers[i]) || (v.name !== old_layers[i].name)){
							if(i < map_layers.getLength())
								map_layers.removeAt(i)

							map_layers.insertAt(i, new Tile({
								source: new TileWMS({
									url: 'https://apps.ecmwf.int/wms',
									params: {LAYERS: v.name, token: 'metview'}
								})
							}))
						}
					})
				}
			})
	},

	set_projection: function(p) {
	},

	set_time: function(time) {
		this.map.getLayers().forEach(function(layer) {
			// TODO don't update the time for non-data layers
			layer.getSource().updateParams({time: time.valid_time.iso8601()})
		})
	},

	_push_undo: function() {
		return this
	},
	_clear: function() {
		return this
	},
	_redraw: function() {
		return this
	},
	_notifyProductChange: function() {
	},
}, mixins)

export default Chart;
